@@ -17,6 +17,8 @@ from ..message import Message
 from ..modloader import trigger_mod_event
 from ..events import trigger_event
 from ..permission import perms
+from ..modloader import Mod
+from ..shared import set_bot
 
 
 # noinspection PyMethodMayBeStatic
@@ -24,8 +26,14 @@ class BaseBot:
     def __init__(self):
         self.irc: Irc = None
         self._running = False
+        set_bot(self)
 
     # region events
+    async def on_mod_reloaded(self, mod: Mod):
+        """
+        triggered when a mod is reloaded using reload_mod() or !reloadmod
+        :param mod: mod being reloaded
+        """
 
     async def on_connected(self):
         """
@@ -173,18 +181,18 @@ class BaseBot:
         ):
             return await msg.reply(
                 whisper=True,
-                msg=f'you do not have permission to execute {cmd.fullname} in #{msg.channel_name}')
+                msg=f'you do not have permission to execute {cmd.fullname} in #{msg.channel_name}, permission required: {cmd.permission}')
 
         elif not isinstance(cmd, CustomCommandAction) and is_command_disabled(msg.channel_name, cmd.fullname):
             return await msg.reply(f'{cmd.fullname} is disabled for this channel')
 
-        if (not perms.has_permission(msg.channel_name, msg.author, cmd.cooldown_bypass) and
+        has_cooldown_bypass_permission = perms.has_permission(msg.channel_name, msg.author, cmd.cooldown_bypass)
+        if (not has_cooldown_bypass_permission and
                 is_command_on_cooldown(msg.channel_name, cmd.fullname, cmd.cooldown)):
             return await msg.reply(
                 f'{cmd.fullname} is on cooldown, seconds left: {cmd.cooldown - get_time_since_execute(msg.channel_name, cmd.fullname)}')
 
-        if (
-                not await self.on_before_command_execute(msg, cmd)
+        if (not await self.on_before_command_execute(msg, cmd)
                 or not all(await trigger_mod_event(Event.on_before_command_execute, msg, cmd, channel=msg.channel_name))
                 or not all(await trigger_event(Event.on_before_command_execute, msg, cmd))
         ):
@@ -192,7 +200,8 @@ class BaseBot:
 
         try:
             await cmd.execute(msg)
-            update_command_last_execute(msg.channel_name, cmd.fullname)
+            if not has_cooldown_bypass_permission:
+                update_command_last_execute(msg.channel_name, cmd.fullname)
         except InvalidArgumentsError as e:
             await self._send_cmd_help(msg, cmd, e)
         else:
