@@ -1,19 +1,25 @@
 import os
+import sys
 import traceback
-from inspect import isclass
+from asyncio import get_event_loop
+from importlib import import_module
+from inspect import isclass, getfile, getmodulename
+from pathlib import Path
+from traceback import print_exc
+from typing import Dict, Callable, Any
 
-from typing import Dict
-from .util import temp_syspath, get_py_files, get_file_name
+from .shared import get_bot
 from .channel import Channel
 from .command import Command
 from .config import cfg
+from .disabled_mods import is_mod_disabled
 from .enums import Event
 from .message import Message
-from .disabled_mods import is_mod_disabled
-from importlib import import_module
+from .util import temp_syspath, get_py_files, get_file_name
+from .events import trigger_event
 
 __all__ = ('ensure_mods_folder_exists', 'Mod', 'register_mod', 'trigger_mod_event', 'mods',
-           'load_mods_from_directory', 'mod_exists')
+           'load_mods_from_directory', 'mod_exists', 'reload_mod', 'is_mod', 'unregister_mod')
 
 
 # noinspection PyMethodMayBeStatic
@@ -33,6 +39,12 @@ class Mod:
         :param channel: the channel the mod is disabled in
         """
 
+    async def on_mod_reloaded(self, mod: 'Mod'):
+        """
+        triggered when a mod is reloaded using reload_mod() or !reloadmod
+        :param mod: mod being reloaded
+        """
+
     async def on_connected(self):
         """
         triggered when the bot connects to all the channels specified in the config file
@@ -96,6 +108,20 @@ class Mod:
         triggered when the bot joins a channel
         """
 
+    async def on_user_join(self, user: str, channel: Channel):
+        """
+        triggered when a user joins a channel the bot is in
+        :param user: the user who joined
+        :param channel: the channel that the user joined
+        """
+
+    async def on_user_part(self, user: str, channel: Channel):
+        """
+        triggered when a user leaves from a channel the bot is in
+        :param user: the user who left
+        :param channel: the channel that the user left
+        """
+
     async def on_channel_subscription(self, channel: Channel, msg: Message):
         """
         triggered when a user subscribes
@@ -120,6 +146,18 @@ def register_mod(mod: Mod) -> bool:
     return True
 
 
+def unregister_mod(mod: Mod) -> bool:
+    """
+    unregisters a mod from the global cache `mods`
+    :param mod: mod to unregister
+    :return: if it successfully unregistered it
+    """
+    if mod.name not in mods:
+        return False
+
+    del mods[mod.name]
+
+
 async def trigger_mod_event(event: Event, *args, channel: str = None) -> list:
     """
     triggers a event on all mods
@@ -136,7 +174,7 @@ async def trigger_mod_event(event: Event, *args, channel: str = None) -> list:
 
     output = []
     for mod in mods.values():
-        if channel and is_mod_disabled(channel, mod.name):
+        if channel is not None and is_mod_disabled(channel, mod.name):
             continue
 
         try:
@@ -160,7 +198,7 @@ def ensure_mods_folder_exists():
         os.mkdir(cfg.mods_folder)
 
 
-def load_mods_from_directory(fullpath):
+def load_mods_from_directory(fullpath, predicate: Callable[[str, Any], bool] = None):
     """
     loads all mods from the given directory, only .py files are loaded
     :param fullpath: the path to search for mods to load
@@ -174,10 +212,47 @@ def load_mods_from_directory(fullpath):
             module = import_module(get_file_name(file))
             for obj in module.__dict__.values():
                 # verify the obj is a class, is a subclass of Mod, and is not Mod class itself
-                if not isclass(obj) or not issubclass(obj, Mod) or obj is Mod:
+                if not is_mod(obj):
                     continue
                 # create a instance of the mod subclass, then register it
-                register_mod(obj())
+                if predicate is None or (predicate is not None and predicate(obj.name, obj)):
+                    register_mod(obj())
+
+
+def reload_mod(mod_name: str):
+    mod = mods.get(mod_name)
+    if mod is None:
+        raise ValueError(f'could not find mod by the name of "{mod_name}"')
+
+    try:
+        # get the module's file path, this is needed to add it to python's import search paths
+        path = Path(getfile(mod.__class__))
+        with temp_syspath(path.parent):
+            # this is needed to make python import the latest version from disk,
+            # python caches imports in sys.modules
+            # doing this removes it from the cache, so the latest version from the disk is imported
+            del sys.modules[getmodulename(path)]
+            # this dict stores the Mod's / global objects in the Mod's .py file
+            # it lets us iterate over its value to check for the Mod that we want to reload
+            for item in __import__(getmodulename(path), locals={}, globals={}).__dict__.values():
+                if is_mod(item) and item.name == mod.name:
+                    unregister_mod(mod)
+                    reloaded_mod = item()
+                    register_mod(reloaded_mod)
+                    # trigger events
+                    get_event_loop().create_task(trigger_mod_event(Event.on_mod_reloaded, reloaded_mod))
+                    get_event_loop().create_task(get_bot().on_mod_reloaded(reloaded_mod))
+                    get_event_loop().create_task(trigger_event(Event.on_mod_reloaded, reloaded_mod))
+                    return True
+
+    except Exception as e:
+        print(f'error trying to reload Mod "{mod.name}", error type: {type(e)}, error: {e}')
+        print_exc()
+    return False
+
+
+def is_mod(obj):
+    return isclass(obj) and issubclass(obj, Mod) and obj is not Mod
 
 
 def mod_exists(mod: str) -> bool:
